# Enunciado:
# Faça uma função que cria uma matriz de dimensões n x m e inicializa cada
# célula com um mesmo valor dado como parâmetro (por padrão zero).

def nova_matriz(n, m, valor=0):
    matriz = []
    for i in range(n):
        linha = []
        for j in range(m):
            linha.append(valor)
        matriz.append(linha)
    return matriz

# Enunciado:
# Faça uma função que cria uma matriz identidade de dimensão n.

# Versão sem usar o nova_matriz


def identidade(n):
    matriz = []
    for i in range(n):
        linha = []
        for j in range(n):
            if i == j:
                linha.append(1)
            else:
                linha.append(0)
        matriz.append(linha)
    return matriz

# Versão usando o nova_matriz


def identidade(n):
    matriz = nova_matriz(n, n)
    for i in range(n):
        matriz[i][i] = 1


# Enunciado:
# Faça uma função que imprime uma matriz

def imprime(matriz):
    for linha in matriz:
        # join não é uma opção aqui pois os elementos podem não ser str
        for i in range(len(linha) - 1):
            print(linha[i], end=" ")
        print(linha[-1])


# Enunciado:
# Faça uma função que multiplica uma matriz por um escalar

# Versão que itera usando range
def multiplica_por_escalar(matriz, escalar):
    nova = []
    for i in range(len(matriz)):
        linha = []
        for j in range(len(matriz[i])):
            resultado = matriz[i][j] * escalar
            linha.append(resultado)
        nova.append(linha)
    return nova


# Versão que itera sem usar o range
def multiplica_por_escalar(matriz, escalar):
    nova = []
    for linha in matriz:
        nova_linha = []
        for celula in linha:
            resultado = celula * escalar
            nova_linha.append(resultado)
        nova.append(nova_linha)
    return nova


# Versão que primeiro criar a matriz e depois preenche
def multiplica_por_escalar(matriz, escalar):
    nova = nova_matriz(len(matriz), len(matriz[0]))
    for i in range(len(matriz)):
        for j in range(len(matriz[i])):
            nova[i][j] = matriz[i][j] * escalar
    return nova

# Enunciado:
# Faça uma função que soma duas matrizes


def soma_matrizes(matriz1, matriz2):
    if len(matriz1) != len(matriz2) \
            or len(matriz1[0]) != len(matriz2[0]):
        raise ValueError("Matrizes de dimensões incompatíveis!")
    nova = nova_matriz(len(matriz1), len(matriz1[0]))
    for i in range(len(matriz1)):
        for j in range(len(matriz1[i])):
            nova[i][j] = matriz1[i][j] + matriz2[i][j]
    return nova


# Enunciado:
# Faça uma função que transpõe uma matriz


def transpoe(matriz):
    nova = nova_matriz(len(matriz[0]), len(matriz))
    for i in range(len(nova)):
        for j in range(len(nova[0])):
            nova[i][j] = matriz[j][i]
    return nova

# Enunciado:
# Faça uma função que multiplica duas matrizes


def multiplicacao(matriz1, matriz2):
    if len(matriz1[0]) != len(matriz2):
        raise ValueError()
    resultado = nova_matriz(len(matriz1), len(matriz2[0]))
    for i in range(len(resultado)):
        for j in range(len(resultado[i])):
            for k in range(len(matriz1[i])):
                resultado[i][j] += matriz1[i][k] * matriz2[k][j]
    return resultado


matriz = nova_matriz(2, 3, 42)
matriz[0][0] = 34
print("matriz")
imprime(matriz)
nova = multiplica_por_escalar(matriz, 2)
print("nova")
imprime(nova)
print("matriz")
imprime(matriz)
imprime(soma_matrizes(matriz, nova))
imprime(transpoe(soma_matrizes(matriz, nova)))
